/* Arduino sketch used for reading/writing data from/to Suburu stock Clarion stereo.  Uses SPI
 *  to mimic VFD display output.  Writing to radio's inputs requires the use of Sanyo's CCB protocol
 *  since the radio input display panel will be entirely removed, it is necessary to send commands to the 
 *  radio's internal controller with this protocol, the radio input panel itself employs a Sanyo LC75700T.  
 *  Using a timing hack within the ISR, it is possible 
 *  to force an output on consistently thereby changing the radio's presets, mode of operation, 
 *  band, etc.. This makes is possible to circumvent reading the clock pulses required for synchronous 
 *  serial communication that is required by the CCB protocol
 *  
 *  
 *  ADDS functionality for BLE Sheild using BlueGiga BLE113
 */
//For Serial comms with BLE Shield
#include <SoftwareSerial.h>

SoftwareSerial BLE_Shield(4,5);  // Configure the Serial port to be on pins D4 and D5. This
                                 // will match the jumpers on the BLE Shield (RX -> D4 & TX /> D5).
//For Rotary Encoder LEDs
#define VOL_KNOB A5
#define MENU_KNOB A3
#define MENU_INPUT 6
#define TONE_KNOB A2
#define OTHER_KNOB A4
 
//For CCB 
#define CCB_CE 3    // LC75700 Radio panel 
//#define CCB_DO 4    // Panel's Data Output pin (goes to radio's DI)
#define CCB_DO 8    // Panel's Data Output pin (goes to radio's DI)

//#define CCB_DI 5    // Panel's Data Input pin (goes to radio's DO)
#define CCB_DI 9    // Panel's Data Input pin (goes to radio's DO)

//assembly code instruction used for extremely short delays (62.5ns on a 16MHz AtMega)
#define NOP __asm__ __volatile__ ("nop\n\t")

//For SPI
#include <SPI.h>

#define SCK_PIN   13
#define MISO_PIN  12
#define MOSI_PIN  11
#define SS_PIN    10

// set up the speed, data order and data mode
SPISettings settingsA(2000000, MSBFIRST, SPI_MODE1);

//int currentAddress = 0;      // Address buffer
int dataIdx = 0;             // Index of the current bit on the data buffer
int inputSet = 3;           // this integer is for setting how many times to send the output, anything over 9 used in setting the radio presets
                            // minimum is 3
char ISR_Delay = '0';       //initialize default delay for ISR
uint8_t I_Test = 0; 
uint8_t I_Test2 = 0;

//SPI Initialization
void SlaveInit(void) {

 pinMode(SCK_PIN, INPUT);
 pinMode(MOSI_PIN, INPUT);
 pinMode(MISO_PIN, OUTPUT);
 pinMode(SS_PIN, INPUT);
 
 /*  Setup SPI control register SPCR
 SPIE - Enables the SPI interrupt when 1
 SPE - Enables the SPI when 1
 DORD - Sends data least Significant Bit First when 1, most Significant Bit first when 0
 MSTR - Sets the Arduino in master mode when 1, slave mode when 0
 CPOL - Sets the data clock to be idle when high if set to 1, idle when low if set to 0
 CPHA - Samples data on the trailing edge of the data clock when 1, leading edge when 0
 SPR1 and SPR0 - Sets the SPI speed, 00 is fastest (4MHz) 11 is slowest (250KHz)
 
 // enable SPI subsystem and set correct SPI mode
 // SPCR = (1<<SPE)|(1<<DORD)|(0<<MSTR)|(0<<CPOL)|(0<<CPHA)|(00<<SPR1)|(00<<SPR0); 
 */
//configuration for communcication with VFD display
SPCR = (1<<SPE)|(1<<DORD)|(0<<MSTR)|(1<<CPOL)|(1<<CPHA)|(00<<SPR1)|(00<<SPR0);
}

//initialize Renesas array
uint8_t Renesas[32] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                      0x00, 0x00, 0x00, 0x00};

//Character Array for OKI L9203-06 VFD Driver (taken from datasheet, zeroes placed for unused characters)
char Renesas_Conv[146] = { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,' ', '!', '"', '#', '$', '%', '&', '`', '(', ')', 
                        '*', '+', ',', '-', '.', '/', '0','1','2','3','4','5','6','7','8','9',':',';','<',
                        '=','>','?','@','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q',
                        'R','S','T','U','V','W','X','Y','Z','[',' ',']','^','_','`','a','b','c','d','e','f',
                        'g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z','{',
                        '|','}','~',' ',0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,'+',0,'\n',0};                 

//function for reading SPI interface
uint8_t ReadByte(void) {
//delayMicroseconds(41);
//if (BLE_Shield.available()) {  
  //SPI.beginTransaction(settingsA);
   for (int i = 0; i < 32; i++){
    Renesas[i] = SPI.transfer(0x00);
   }
  
 //  BLE_Shield.print("SS");
 // SPI.endTransaction;   
 // }
}


void setup() {
  //CCB setup below
  //re-initialize Data Index   
  dataIdx = 0;

   pinMode(MENU_KNOB, OUTPUT);
   pinMode(VOL_KNOB, OUTPUT);
   pinMode(MENU_INPUT, OUTPUT);
   pinMode(OTHER_KNOB, INPUT_PULLUP);
   pinMode(TONE_KNOB, INPUT_PULLUP);
  
  //set up interrupt for when the CE line goes high momentarily to initiate data transfer
  // '1' interrupts pin 3 on Uno
  attachInterrupt(1, onCE, RISING);

  //setup LEDs to be on
  digitalWrite(MENU_KNOB, HIGH);
  digitalWrite(VOL_KNOB, HIGH);

  //set DO to an output and pull it low to match datasheet
  pinMode(CCB_DO, OUTPUT);
  digitalWrite(CCB_DO, LOW);

  //set DI to read controller's DO for address (NOT USED)
  pinMode(CCB_DI, INPUT);
  //end of CCB setup
  
  //set up serial comms
  //Serial.begin(9600);

  BLE_Shield.begin(9600);   // Setup the serial port at 9600 bps. This is the BLE Shield default baud rate.

  //Call SlaveInit() for the SPI communication used for the VFD output
  SlaveInit();
 
  BLE_Shield.write(254); // cursor to beginning of first line
  BLE_Shield.write(128);

  //  BLE_Shield.write("                "); // clear display + legends
 // BLE_Shield.write("                ");

}

void loop() {

bool OTHER_state =  digitalRead(OTHER_KNOB);
bool TONE_state =  digitalRead(TONE_KNOB);

  //Reads SPI interface every loop

  ReadByte();

  for (int j = 0;j<32;j++){  // note loop values
  //below formats the SPI output to mimic the VFD display
  
  //print top line output of VFD display
  if (Renesas[j] == 0xF0){
    j=j+2;
      for (int k = 0;k<12;k++){
      BLE_Shield.print(Renesas_Conv[Renesas[j]]);    
  
      j++;
    }

  j=j+1;
  //print out new line character
  for (int k = 0;k<12;k++){
  BLE_Shield.print(Renesas_Conv[Renesas[j]]);    
  j++;
  }  
 }
 
    else if (Renesas[j] == 0x3C){
    }
 
    else {
    }
  }

//reset Renesas character array
memset(Renesas, 0, sizeof(Renesas));
//memset(VFD_Message_Bot_Line, 0, sizeof(VFD_Message_Bot_Line));
 
     
  //Increment Data Index
  if (dataIdx >= inputSet){
  dataIdx = 0;
  ISR_Delay = '0';
  }

  if (ISR_Delay != '0'){
  dataIdx++;
  }

  if(TONE_state == LOW){
    digitalWrite(VOL_KNOB, LOW);
            inputSet = 1;
    ISR_Delay = 'a';      //set satellite button with knob press
    onCE();
    delay(200); 
    digitalWrite(VOL_KNOB, HIGH);
  }
  if(OTHER_state == LOW){
    digitalWrite(MENU_KNOB, LOW);
    digitalWrite(MENU_INPUT, HIGH);
    delay(100);
    digitalWrite(MENU_INPUT, LOW);
    delay(100);
    digitalWrite(MENU_INPUT, HIGH);
    digitalWrite(MENU_KNOB, HIGH);

  }

if (BLE_Shield.available() > 0) {
 // See if new position data is available
  I_Test = BLE_Shield.read();
  
  //Test the inputs if a match set the ISR_Delay to the matching string from the iPhone App
   if (I_Test == 1){
    // preset 1
    inputSet = 1;
    ISR_Delay = '1';
   }
   else if (I_Test == 2){
    // preset 2
    inputSet = 1;
    ISR_Delay = '2';
   }
   else if (I_Test == 3){
        //preset 3
        inputSet = 1;
    ISR_Delay = '3';
   }
   else if (I_Test == 4){
      //preset 4
        inputSet = 1;
    ISR_Delay = '4';
   }
   else if (I_Test == 5){
        //preset 5
        inputSet = 1;
    ISR_Delay = '5';
   }
   else if (I_Test == 6){
        //preset 6
        inputSet = 1;
    ISR_Delay = '6';
   }
   else if (I_Test == 7){
        // fm/am
        inputSet = 1;
    ISR_Delay = 'f';
   }
   else if (I_Test == 8){
       // CD/AUX
       inputSet = 1;
    ISR_Delay = 'c';
   }
   else if (I_Test == 9){
        // Menu
        inputSet = 1;
    ISR_Delay = 'm';
   }
   else if (I_Test == 10){
        //Text
        inputSet = 1;
    ISR_Delay = 't';
   }
   else if (I_Test == 11){
        //Scan
        inputSet = 1;
    ISR_Delay = 's';
   } 
    else if (I_Test == 13){
        //SAT
        inputSet = 1;
    ISR_Delay = 'a';
   }
    if (I_Test == 14){
      //"1"
   inputSet = 9;
    ISR_Delay = '1';
   }
   else if (I_Test == 15){
    //"2"
    inputSet = 9;
    ISR_Delay = '2';
   }
   else if (I_Test == 16){
    //"3"
    inputSet = 9;
    ISR_Delay = '3';
   }
   else if (I_Test == 17){
    //"4"
    inputSet = 9;
    ISR_Delay = '4';
   }
   else if (I_Test == 18){
    // "5"
    inputSet = 9;
    ISR_Delay = '5';
   }
   else if (I_Test == 19){
    // "6"
    inputSet = 9;
    ISR_Delay = '6';
   }
   else{

   }

 }

}


 
void onCE() {

  if (dataIdx < inputSet){
 
  switch(ISR_Delay) {
    case ('1'):
    delayMicroseconds(41);
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    digitalWrite(CCB_DO, HIGH); //writes output high and then low
    digitalWrite(CCB_DO, LOW);
    break;

    case ('2'):
    delayMicroseconds(19);
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    digitalWrite(CCB_DO, HIGH);
    digitalWrite(CCB_DO, LOW);
    break;
    
    case ('3'):
    delayMicroseconds(22);
    NOP;
    NOP;

    digitalWrite(CCB_DO, HIGH); //writes output high and then low
    digitalWrite(CCB_DO, LOW);
    break;

    case ('4'):
    delayMicroseconds(25);
    NOP;
    NOP;
    NOP;
    NOP;

    digitalWrite(CCB_DO, HIGH); //writes output high and then low
    digitalWrite(CCB_DO, LOW);
    break;

    case ('5'):
    delayMicroseconds(27);
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    digitalWrite(CCB_DO, HIGH); //writes output high and then low
    digitalWrite(CCB_DO, LOW);
    break;

    case ('6'):
    delayMicroseconds(30);
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;

    digitalWrite(CCB_DO, HIGH); //writes output high and then low
    digitalWrite(CCB_DO, LOW);
    break;

    //case for "FM/AM" mode
    case ('f'):
    delayMicroseconds(33);
    NOP;
    NOP;
    NOP;
    digitalWrite(CCB_DO, HIGH); //writes output high and then low
    digitalWrite(CCB_DO, LOW);
    break;

    //case for left arrow
    case ('l'):
    NOP;
    NOP;
    NOP;
    NOP;
     NOP;
    NOP;
    NOP;
    digitalWrite(CCB_DO, HIGH); //writes output high and then low
    digitalWrite(CCB_DO, LOW);
    break;

    //case for right arrow
    case ('r'):
    delayMicroseconds(1);
    digitalWrite(CCB_DO, HIGH); //writes output high and then low
    digitalWrite(CCB_DO, LOW);
    break;

    //case for down arrow
    case ('u'):

    NOP;
    NOP;
    NOP;
     NOP;
    NOP;
    NOP;
     NOP;
    NOP;
    NOP;
    digitalWrite(CCB_DO, HIGH); //writes output high and then low
    digitalWrite(CCB_DO, LOW);
    break;

    //case for left arrow
    case ('d'):
     NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    digitalWrite(CCB_DO, HIGH); //writes output high and then low
    digitalWrite(CCB_DO, LOW);
    break;

    //case for "MENU" button
    case ('m'):
    delayMicroseconds(13);
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    digitalWrite(CCB_DO, HIGH); //writes output high and then low
    digitalWrite(CCB_DO, LOW);
    break;

    //case for "CD/AUX" button
    case ('c'):
    delayMicroseconds(40);  //unknown
    NOP;
    NOP;
    NOP;
    digitalWrite(CCB_DO, HIGH); //writes output high and then low
    digitalWrite(CCB_DO, LOW);
    break;

    //case for "SCAN" button
    case ('s'):
    delayMicroseconds(10);
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    
    digitalWrite(CCB_DO, HIGH); //writes output high and then low
    digitalWrite(CCB_DO, LOW);
    break;

    //case for "TEXT" button
    case ('t'):
    delayMicroseconds(4);
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
    NOP; 
    NOP;
    NOP;
    NOP;
    NOP;
    NOP;
   
    digitalWrite(CCB_DO, HIGH); //writes output high and then low
    digitalWrite(CCB_DO, LOW);
    break;

    //case for "SAT" button
    case ('a'):
    delayMicroseconds(36);
    digitalWrite(CCB_DO, HIGH); //writes output high and then low
    digitalWrite(CCB_DO, LOW);
    break;
    
      }
    }

}

